//---------------------------------------------------------------------------

#ifndef Main_FormH
#define Main_FormH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ExtCtrls.hpp>
#include <windows.h>
#include <mmsystem.h>
#include <stdlib.h>
#include <Grids.hpp>

const int C_PERIOD=5;   // clock period for threads

// high resolution timer object, Win32 Version
class THighResTimer
{
private:
	LARGE_INTEGER t0,t1,f;
	double et,last_dt;
	bool notActive;
protected:
public:
	THighResTimer()
	{
		QueryPerformanceFrequency(&f);
		notActive=f.QuadPart==0;
		HighResReset();
	}

	void HighResReset()   // reset timer
	{
		if (!notActive) QueryPerformanceCounter(&t0);
		last_dt=0.0;
		et=0.0;
	}

	double HighResTime()  // elapsed time in seconds
	{
		double dt;

		if (notActive) return 0.0;

		QueryPerformanceCounter(&t1);
		dt=t1.QuadPart-t0.QuadPart;
		if (dt<0.0)
		{
			et=et+last_dt/f.QuadPart;
			t0=t1;
			last_dt=0.0;
			return et;
		}
		else
		{
			last_dt=dt;
			return et+dt/f.QuadPart;
		}
	}
};

class TCrashThread:public TThread
{
private:
  // these allow monitoring of operation
  unsigned int RunCount;
	THighResTimer HRT;
	double RunSeconds;
protected:
	virtual void __fastcall Execute(void);
public:
	TCrashThread *pullThread;	// thread to copy string from

	AnsiString s1;	// our string
	AnsiString s2;	// copied string from pullThread

	__fastcall TCrashThread(void);

	// call to start the thread
  virtual void StartThread(TThreadPriority priority);

	// call to stop the thread
  virtual void StopThread(void);

	// call to log start run event
  virtual void StartRunEvent(void);

	// call to log stop run event
  virtual void StopRunEvent(void);

	// methods for thread monitoring
  unsigned int GetRunCount(void) { return RunCount; };
	double GetRunSeconds(void) { return RunSeconds; };
};

const int NTHREAD=11;	// number of threads to run

//---------------------------------------------------------------------------
class TForm1 : public TForm
{
__published:	// IDE-managed Components
  TTimer *Timer1;
  TStringGrid *StringGrid1;
  void __fastcall Timer1Timer(TObject *Sender);
  void __fastcall FormShow(TObject *Sender);
  void __fastcall FormClose(TObject *Sender, TCloseAction &Action);
private:	// User declarations
public:		// User declarations

	TCrashThread *crashThreads[NTHREAD];

	double maxTime[NTHREAD];

  __fastcall TForm1(TComponent* Owner);
};
//---------------------------------------------------------------------------
extern PACKAGE TForm1 *Form1;
//---------------------------------------------------------------------------
#endif
